'use strict';

const _ = require('lodash');
const assert = require('assert');

module.exports = class AssertionUtils {
    static assertEach (asserters) {
        assert(_.isArray(asserters));
        _.forEach(asserters, assertValid);

        function assertValid (asserter) {
            AssertionUtils.assertGeneric(...asserter);
        }
    }

    static assertGeneric (name, value, validator) {
        if (_.isFunction(validator)) {
            const validated = validator(value);

            return assert(validated, `${name} validation failed. Expected: "true". Actual: "${validated}".`);
        } else if (_.isObject(validator)) {
            return AssertionUtils.assertSchema(value, validator);
        }

        return assert.strictEqual(validator, value,
            `${name} validation failed. Expected: ${validator}. Actual: ${value}.`);
    }

    static assertObjectPropertiesAreAtLeast (object, expectedList = []) {
        assert(_.isObject(object));

        const actualList = _.keys(object);

        const objectHasAllRequiredProperties = _.isEmpty(_.difference(expectedList, actualList));
        assert(objectHasAllRequiredProperties, 'Missing expected properties on object.');
    }

    static assertObjectPropertiesAreExactly (object, expectedList = []) {
        assert(_.isObject(object));

        const actualList = _.keys(object);

        const objectHasExactSameProperties = _.isEqual(expectedList, actualList);
        assert(objectHasExactSameProperties, 'Object properties do not match.');
    }

    static assertObjectPropertiesAreAtMost (object, expectedList = []) {
        assert(_.isObject(object));

        const actualList = _.keys(object);

        const objectHasNoExtraProperties = _.isEmpty(_.difference(actualList, expectedList));
        assert(objectHasNoExtraProperties, 'Additional unexpected properties on object.');
    }

    static assertPropertyIs (object, propertyName, expectedValue) {
        assert(_.has(object, propertyName), 'Object was missing the property.');
        assert.deepStrictEqual(expectedValue, _.get(object, propertyName));
    }

    static assertSchema (object, schema, { strict = true } = {}) {
        assert(_.isObject(object));
        if (strict) AssertionUtils.assertObjectPropertiesAreAtMost(object, _.keys(schema));
        _.forEach(schema, assertValid);

        function assertValid (validator, propertyName) {
            const propertyValue = object[propertyName];
            AssertionUtils.assertGeneric(propertyName, propertyValue, validator);
        }
    }
};
