/* eslint-disable
 func-names,
 func-style,
 max-nested-callbacks,
 no-undefined,
 prefer-arrow-callback,
 */
/* global describe, it */

'use strict';

const _ = require('lodash');
const appModulePath = require('app-module-path');
const assert = require('assert');

appModulePath.addPath('./');

const AssertionUtils = require('lib/assertionUtils');

describe('AssertionUtils', function () {
    describe('AssertionUtils::assertEach', function () {
        it('should succeed for congruent elements', function () {
            AssertionUtils.assertEach([
                ['bar', 42, 42],
                ['foo', { baz: true }, { baz: true }],
                ['qux', null, _.isNil],
            ]);
        });

        it('should fail for an invalid element', function () {
            assert.throws(() => AssertionUtils.assertEach([
                ['bar', 'string', 42],
                ['foo', { baz: true }, { baz: true }],
                ['qux', null, _.isNil],
            ]));
        });
    });

    describe('AssertionUtils::assertGeneric', function () {
        it('should succeed when a funcion returns truthy', function () {
            const validator = input => 'functionInput' === input;
            AssertionUtils.assertGeneric('myName', 'functionInput', validator);
        });

        it('should fail when a funcion returns falsey', function () {
            const validator = input => 'functionInput' === input;
            assert.throws(() => AssertionUtils.assertGeneric('myName', 'aWrongInput', validator));
        });

        it('should succeed when two primitives are the same', function () {
            AssertionUtils.assertGeneric('myName', 'primitive', 'primitive');
        });

        it('should fail when two primitives are different', function () {
            assert.throws(() => AssertionUtils.assertGeneric('myName', 'primitive', 'differentPrimitive'));
        });

        it('should fail when two primitives are the same, but different types', function () {
            assert.throws(() => AssertionUtils.assertGeneric(42, '42', 'differentPrimitive'));
        });

        it('should succeed when two objects are the same', function () {
            const object1 = { bar: 42, foo: { baz: true } };
            const object2 = _.cloneDeep(object1);
            AssertionUtils.assertGeneric('myName', object1, object2);
        });

        it('should fail when two objects are different (same keys, different values)', function () {
            const object1 = { bar: 42, foo: { baz: true } };
            const object2 = _.cloneDeep(object1);
            _.set(object2, 'foo.baz', false);
            assert.throws(() => AssertionUtils.assertGeneric('myName', object1, object2));
        });

        it('should fail when two objects are different (missing a key)', function () {
            const object1 = { bar: 42, foo: { baz: true } };
            const object2 = _.cloneDeep(object1);
            _.unset(object2, 'foo.baz');
            assert.throws(() => AssertionUtils.assertGeneric('myName', object1, object2));
        });

        it('should fail when two objects are different (additional key)', function () {
            const object1 = { bar: 42, foo: { baz: true } };
            const object2 = _.cloneDeep(object1);
            _.set(object2, 'foo.qux', true);
            assert.throws(() => AssertionUtils.assertGeneric('myName', object1, object2));
        });
    });

    describe('AssertionUtils::assertObjectPropertiesAreAtLeast', function () {
        it('should succeed when object properties are correct', function () {
            const object = { bar: 42, foo: { baz: true } };
            AssertionUtils.assertObjectPropertiesAreAtLeast(object, ['bar', 'foo']);
        });

        it('should succeed when there are additional properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            AssertionUtils.assertObjectPropertiesAreAtLeast(object, ['foo']);
        });

        it('should fail when there are missing properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            assert.throws(() => AssertionUtils.assertObjectPropertiesAreAtLeast(object, ['bar', 'baz', 'foo']));
        });
    });

    describe('AssertionUtils::assertObjectPropertiesAreExactly', function () {
        it('should succeed when object properties are correct', function () {
            const object = { bar: 42, foo: { baz: true } };
            AssertionUtils.assertObjectPropertiesAreExactly(object, ['bar', 'foo']);
        });

        it('should fail when there are additional properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            assert.throws(() => AssertionUtils.assertObjectPropertiesAreExactly(object, ['foo']));
        });

        it('should fail when there are missing properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            assert.throws(() => AssertionUtils.assertObjectPropertiesAreExactly(object, ['bar', 'baz', 'foo']));
        });
    });

    describe('AssertionUtils::assertObjectPropertiesAreAtMost', function () {
        it('should succeed when object properties are correct', function () {
            const object = { bar: 42, foo: { baz: true } };
            AssertionUtils.assertObjectPropertiesAreAtMost(object, ['bar', 'foo']);
        });

        it('should succeed when there are missing properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            AssertionUtils.assertObjectPropertiesAreAtMost(object, ['bar', 'baz', 'foo']);
        });

        it('should fail when there are additional properties', function () {
            const object = { bar: 42, foo: { baz: true } };
            assert.throws(() => AssertionUtils.assertObjectPropertiesAreAtMost(object, ['foo']));
        });
    });

    describe('AssertionUtils::assertPropertyIs', function () {
        it('should succeed for a number property', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                qux: undefined,
            };
            AssertionUtils.assertPropertyIs(object, 'bar', 42);
        });

        it('should succeed for an object property', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                qux: undefined,
            };
            AssertionUtils.assertPropertyIs(object, 'foo', _.cloneDeep(object.foo));
        });

        it('should succeed for an undefined property', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                qux: undefined,
            };
            AssertionUtils.assertPropertyIs(object, 'qux', undefined);
        });

        it('should fail for a missing property', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                qux: undefined,
            };
            assert.throws(() => AssertionUtils.assertPropertyIs(object, 'quux', undefined));
        });
    });

    describe('AssertionUtils::assertSchema', function () {
        it('should succeed for a proper schema', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                qux: null,
            };

            const schema = {
                bar: 42,
                foo: { baz: true },
                qux: _.isNil,
            };

            AssertionUtils.assertSchema(object, schema);
        });

        it('should fail for a proper schema with additional properties', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                quux: 'string',
                qux: null,
            };

            const schema = {
                bar: 42,
                foo: { baz: true },
                qux: _.isNil,
            };

            assert.throws(() => AssertionUtils.assertSchema(object, schema));
        });

        it('should succeed for a proper schema with additional properties, if "strict" is false', function () {
            const object = {
                bar: 42,
                foo: { baz: true },
                quux: 'string',
                qux: null,
            };

            const schema = {
                bar: 42,
                foo: { baz: true },
                qux: _.isNil,
            };

            AssertionUtils.assertSchema(object, schema, { strict: false });
        });

        it('should fail for an invalid property', function () {
            const object = {
                bar: 'string',
                foo: { baz: true },
                qux: null,
            };

            const schema = {
                bar: 42,
                foo: { baz: true },
                qux: _.isNil,
            };

            assert.throws(() => AssertionUtils.assertSchema(object, schema));
        });

        it('should fail for a missing property', function () {
            const object = {
                bar: 'string',
                foo: { baz: true },
            };

            const schema = {
                bar: 42,
                foo: { baz: true },
                qux: _.isNull,
            };

            assert.throws(() => AssertionUtils.assertSchema(object, schema));
        });
    });
});
